from uuid import uuid4
from blockchain import *
import json
from django.http import JsonResponse

node_identifier = str(uuid4()).replace('-', '')

algoablockchain = AlgoaBlockchain()


# Create your views here.
def mine(request):
    # We run the proof of work algorithm to get the next proof...
    last_block = algoablockchain.last_block

    last_proof = last_block['proof']
    proof = algoablockchain.proof_of_work(last_proof)

    # We must receive a reward for finding the proof.
    # The sender is "0" to signify that this node has mined a new coin.
    algoablockchain.new_transaction(
        sender="0",
        recipient=node_identifier,
        amount=1,
    )

    # Forge the new Block by adding it to the chain
    previous_hash = algoablockchain.hash(last_block)
    block = algoablockchain.new_block(proof, previous_hash)

    response = {
        'message': "New Block Forged",
        'index': block['index'],
        'transactions': block['transactions'],
        'proof': block['proof'],
        'previous_hash': block['previous_hash'],
    }
    return JsonResponse(response)


def new_transaction(request):
    values = json.loads(request.body)

    # Check that the required fields are in the POST'ed data
    required = ['sender', 'recipient', 'amount']
    if not all(k in values for k in required):
        return 'Missing values', 400

    # Create a new Transaction
    index = algoablockchain.new_transaction(values['sender'], values['recipient'], values['amount'])

    response = {'message': 'Transaction will be added to Block ' + index}
    return JsonResponse(response)


def full_chain(request):
    response = {
        'chain': algoablockchain.chain,
        'length': len(algoablockchain.chain),
    }
    return JsonResponse(response)


def register_nodes(request):
    values = json.loads(request.body)
    nodes = values.get('nodes')
    if nodes is None:
        return "Error: Please supply a valid list of nodes", 400

    for node in nodes:
        algoablockchain.register_node(node)

    response = {'message': 'New nodes have been added', 'total_nodes': list(blockchain.nodes)}
    return JsonResponse(response)


def consensus(request):
    replaced = algoablockchain.resolve_conflicts()

    if replaced:
        response = {
            'message': 'Our chain was replaced',
            'new_chain': algoablockchain.chain
        }
    else:
        response = {
            'message': 'Our chain is authoritative',
            'chain': algoablockchain.chain
        }

    return JsonResponse(response)
